import "./stringExtensions";
import { GenerateApiResponse } from "./responseTypes";

const MaxErrorMessageLength = 4_000;

export abstract class BaseApiClient {
    constructor(private readonly generateConnector: string) {
        this.generateConnector = generateConnector;
    }

    protected async internalFetch<ResponseType extends GenerateApiResponse>(
        endpoint: string,
        body: string,
        headers?: HeadersInit,
    ): Promise<ResponseType> {
        const url = this.generateConnector.extendUri(endpoint);
        const request: RequestInit = {
            method: "POST",
            body: body,
        };
        if (headers) {
            request.headers = headers;
        }

        const response = await fetch(url, request);
        return await this.validateResponse(response);
    }

    private async validateResponse<ResponseType extends GenerateApiResponse>(response: Response): Promise<ResponseType> {
        if (!response.ok) {
            let resultBody = await response.text();
            if (resultBody.length > MaxErrorMessageLength) {
                resultBody = resultBody.substring(0, MaxErrorMessageLength) + "...";
            }
            throw new Error(`Generate Api Client Error: HTTP Code ${response.status} - '${resultBody}'.`);
        }

        const responseJson = await response.json();

        if (responseJson.error) {
            throw new Error(`Validation error: '${JSON.stringify(responseJson)}'.`);
        }

        return responseJson;
    }
}
