import { BaseApiClient } from "./baseApiClient";
import { Endpoints } from "./endpoints";
import { CreateWorkingFolderResponse, GenerateApiResponse, StartBatchJobResponse } from "./responseTypes";

export class GenerateApiClient extends BaseApiClient {
    constructor(generateConnector: string) {
        super(generateConnector);
    }

    /**
     * Creates a working folder for your batch job.
     * @param name Name of the working folder.
     * @param expiration [Optional] Expiration date of the working folder.
     * @returns Response from the cloud containing a workingFolderId parameter of the created working folder.
     */
    async createWorkingFolder(name: string, expiration?: Date): Promise<CreateWorkingFolderResponse> {
        const endpoint = Endpoints.CreateWorkingFolder;
        const body = JSON.stringify({
            name: name,
            expiration: expiration?.toISOString(),
        });

        return await this.internalFetch<CreateWorkingFolderResponse>(endpoint, body);
    }

    /**
     * Increases the hit count for the specified processing pipeline. After a specific number of hits (set in GUI), the pipeline is triggered.
     * @param pipelineName Name of the processing pipeline.
     * @returns General response from the cloud.
     */
    async increaseHitCount(pipelineName: string): Promise<GenerateApiResponse>;
    /**
     * Increases the hit count for the specified processing pipeline. After a specific number of hits (set in GUI), the pipeline is triggered.
     * This method is compatible with Quadient Cloud versions 23.08 and later.
     * @param pipelineName Name of the processing pipeline.
     * @param increaseAmount Amount of the hit count increase.
     * @returns General response from the cloud.
     */
    async increaseHitCount(pipelineName: string, increaseAmount: number): Promise<GenerateApiResponse>;

    async increaseHitCount(pipelineName: string, increaseAmount?: number): Promise<GenerateApiResponse> {
        const endpoint = Endpoints.IncreaseHitCount;
        const body = JSON.stringify({
            pipelineName: pipelineName,
            increaseAmount: increaseAmount,
        });

        return await this.internalFetch<GenerateApiResponse>(endpoint, body);
    }

    /**
     * Starts a batch job using a defined processing pipeline.
     * @param pipelineName Name of the processing pipeline.
     * @param workingFolderId [Optional] Identifier of a custom working folder. If specified, the processing pipeline will run using the custom working folder.
     * @returns Response from the cloud containing a batchJobId parameter of the created batch job.
     */
    async startBatchJob(pipelineName: string, workingFolderId?: string): Promise<StartBatchJobResponse> {
        const endpoint = Endpoints.StartBatchJob;
        const body = JSON.stringify({
            pipelineName: pipelineName,
            workingFolderId: workingFolderId,
        });

        return await this.internalFetch<StartBatchJobResponse>(endpoint, body);
    }
}
